/*
 *		Obtained from runtime system information and documented by
 *		Joerg Jacobsen, JACOBSEN Software-Entwicklung
 *
 *      Copyright (c) 1992, NeXT Computer, Inc.  All rights reserved.
 */ 
@interface DBBasicAdaptor : Object		// Protocol ???
{
	NXBundle *bundle;
}
- initForDatabase:(DBDatabase *)aDatabase;
/*
 * Creates and sets up bundle object. Its principal class will be have the
 * name of the loadable executable in the bundle directory. Furthermore, ...
 */
- (BOOL)areTransactionsEnabled;
/* 
 */
- beginTransaction;
- bundle;
- commitTransaction;
- (BOOL)connectUsingString:(const char *)loginString;

- (const unsigned char *)connectionName;
/* Returns the name of the adaptor's current connection to a database,
 * or NULL if the adaptor is not connected to a database.  When an adaptor
 * establishes a connection to a database, it records the name of the
 * database to which it is connected.
 */

- contextClass;
//- contextForObject:???;

- (const unsigned char *)currentLoginString;
/* Returns the text of the current login string, or NULL if none has been set. 
 */
- (BOOL)fetchData:aBinder;
/* Every adaptor is required to override either this method or the
 * corresponding method in DBBasicAdaptorContext. It is supposed to set
 * the current record object in the application via the valueForProperty
 * method of DBBinder and any setXXXValue methods of DBValue. DBBinder
 * takes care itself of allocating a new record before sending a "fetchData"
 * message to its DBDatabase object (which in turn forwards this message
 * to its adaptor).
 */
- fetchDone:aBinder;
/* The adaptor is notified that a fetch sequence has been completed. You might
 * want to override this method in an adaptor subclass to free resources
 * no longer needed.
 */
- selectData:aBinder;
- insertData:aBinder;
- updateData:aBinder;
- deleteData:aBinder;
/* constructs a standard SQL statement using the properties and the
 * optional qualifier of aBinder and then sends an evaluateString:Using:
 * message to itself.
 */

- disconnectUsingString:(char *)aString;
- enableTransactions:(BOOL)status;
- (List *)enumerateEntities:(List *)entities;
- enumerateProperties:(List *)properties forEntity:anEntity;

- (BOOL)evaluateString:(const unsigned char *)aString using:aBinder;
/* Every adaptor is required to override either this method or the
 * corresponding method in DBBasicAdaptorContext. aString is interpreted
 * as a standard SQL statement.  Returns the status of the evaluation.
 * The implementation of this method is
 * closely related to the one of "fetchData": the later will access resources 
 * allocated by the former in case the string to be evaluated is concerned 
 * with data retrieval.
 */
- expressionClass;
- (BOOL)isConnected;
- isTransactionInProgress;
- maintainContexts:(BOOL)status;
- outerJoinIsDefault;
- recordIdentityProperty;
- releaseContextForObject: anObject;
- rollbackTransaction;

@end
