
(setq clman-develop nil)

(setq next-clman-doc-directory 
  "/NextLibrary/Documentation/CommonLisp/CLUserMan/")

(defun clman:create-doc-dirs ()
  (interactive)
    (setq default-directory next-clman-code-directory)
    (let ((com "")
	  (dir nil)
	  (dirs '(
		  "manual"
		  "manual/compiler"
		  "manual/foreign"
		  "manual/toplevel"
		  "manual/objc"
		  "manual/abouts"
		  "manual/mp"
		  "manual/excl"
		  "manual/sys"
		  )))
      (if (not (file-writable-p next-clman-code-directory))
	  (error "%s not writable" next-clman-code-directory))
      (while dirs
	(setq dir (car dirs))
	      (if (not (file-exists-p dir))
		  (setq com (concat com dir " ")))
	(setq dirs (cdr dirs)))
	
      (message (concat "Making: " com))
    (shell-command 
     (concat "mkdir " com))))

(let ((file (concat next-clman-code-directory "next-clman.data")))
  (if (not (file-exists-p file))
      (progn (load-file  
	      (concat next-clman-code-directory "next-cook-data.el"))
	     (clman:create-doc-dirs)
	     (cook-up-data))))
(load-file (concat next-clman-code-directory "next-clman.data"))

(defun next::clman-find-file  (doc-file buf)
  (if next:save-clman-pages
      (let* ((package-dir-and-file
	      (substring doc-file 
			 (length next-clman-doc-directory)
			 ))
	     (to (string-match "\.lispd" package-dir-and-file))
	     (old-root (substring package-dir-and-file 0 to))
	     (new-name (concat next-clman-code-directory
			       "manual/"
			       old-root 
			       ".doc")))
	(if (file-exists-p new-name)
	    (progn (switch-to-buffer buf)
		   (insert-file new-name))
	  (let ((com    (concat 
			 "nroff -mFranz " doc-file)))
	    (switch-to-buffer buf)
	    (shell-command   com)
	    (switch-to-buffer buf)
	    (insert-buffer "*Shell Command Output*")
	    (clean-buffer-of-roff)
	    (write-file new-name))))
    (let ((com (concat 
		"nroff -mFranz " doc-file)))
      (switch-to-buffer buf)
      (shell-command   com)
      (switch-to-buffer buf)
      (insert-buffer "*Shell Command Output*")
      (clean-buffer-of-roff))))

(defun next::setup-info-and-oblist ()
  (setq fi:clman-displaying-function 'next::clman-find-file)
  (if (or (not (boundp 'fi::clman-oblist)) 
	  (if (boundp 'fi:clman-version)
	      (not (eq fi:clman-version 'NeXT))))
      (progn 
	(load-file (concat next-clman-code-directory "next-clman.data"))
	(setq  fi:clman-version 'NeXT)))
  (setq 
      fi:clman-package-info 
    (list 
     (list "mp" (concat next-clman-doc-directory "mp/") )
     (list "about" (concat next-clman-doc-directory "abouts/") )
     (list "compiler" (concat next-clman-doc-directory "compiler/") )
     (list "excl" (concat next-clman-doc-directory "excl/") )
     (list "objc" (concat next-clman-doc-directory "objc/") )
     (list "foreign" (concat next-clman-doc-directory "foreign/") )
     (list "sys" (concat next-clman-doc-directory "sys/") )
     (list "tpl" (concat next-clman-doc-directory "toplevel/"))))
  (next::setup-oblist))

(defun next::setup-oblist ()
  (let ((clman-package-nicknames nil)
	(clman-oblist-names nil))
    (setq clman-package-nicknames
      (mapcar 'car fi:clman-package-info))
    (setq clman-oblist-names
      (let ((lis clman-package-nicknames)
	    (oblist-name)
	    (result nil))
	(while lis 
	  (setq oblist-name 
	    (car (read-from-string 
 		  (concat "fi::clman-" (car lis) "-oblist"))))
	  (setq result (cons oblist-name result))
	  (setq lis (cdr lis)))
	result))
    (setq fi::clman-oblist (let ((result nil) (lis clman-oblist-names))
			     (while lis
			       (setq result (append (eval (car lis)) result))
			       (setq lis (cdr lis)))
			     result))))

(defun clean-buffer-of-roff ()
  (interactive)
  ;; get rid of backspace chars, underline chars, 9 chars
  (goto-char (point-min))
  (let ((done nil))
    (while (not done)
      (setq done (not (search-forward "_" (point-max) t)))
      (if (not done)
	  (upcase-word 1))))
  (goto-char (point-min))      
  (replace-string "" "")
  (goto-char (point-min))
  (replace-string "_" "")
  (goto-char (point-min))
  (get-rid-of-page-break-stuff sym)
  (delete-other-windows)
  (if buffer-read-only (toggle-read-only)))


(defun get-rid-of-page-break-stuff (sym)
  (if (search-forward "9" (point-max) t)
      (progn
	(goto-char (point-min))
	(replace-string "9" "")
	(goto-char (point-min))
	;;(replace-regexp (concat "^" sym) "")
	(goto-char (point-min))
	(replace-regexp "^ *1$" ""))))


;;  We set up the table
(next::setup-info-and-oblist)
